#!/usr/bin/env python

from distutils.core import setup, gen_usage, Command, grok_environment_error
from distutils.dist import Distribution
from distutils.errors import *
import os
import sys
try:
  import py2exe
  build_exe = py2exe.build_exe.py2exe
  Distribution = py2exe.Distribution
except ImportError:
  py2exe = None
  build_exe = Command
import wxPRE


class InnoScript:
    """class that builds an InnoSetup script"""
    def __init__(self, name, lib_dir, dist_dir, windows_exe_files = [], lib_files = [], version = "1.0"):
        self.lib_dir = lib_dir
        self.dist_dir = dist_dir
        if not self.dist_dir.endswith(os.sep):
            self.dist_dir += os.sep
        self.name = name
        self.version = version
        self.windows_exe_files = [self.chop(p) for p in windows_exe_files]
        self.lib_files = [self.chop(p) for p in lib_files]

    def chop(self, pathname):
        """returns the path relative to self.dist_dir"""
        assert pathname.startswith(self.dist_dir)
        return pathname[len(self.dist_dir):]

    def create(self, pathname=None):
        """creates the InnoSetup script"""
        if pathname is None:
          self.pathname = os.path.join(self.dist_dir, self.name + os.extsep + "iss")
        else:
          self.pathname = pathname
        ofi = self.file = open(self.pathname, "w")
        print >> ofi, "; WARNING: This script has been created by py2exe. Changes to this script"
        print >> ofi, "; will be overwritten the next time py2exe is run!"
        print >> ofi, r"[Setup]"
        print >> ofi, r"AppName=%s" % self.name
        print >> ofi, r"AppVerName=%s %s" % (self.name, self.version)
        print >> ofi, r"DefaultDirName={pf}\%s" % self.name
        print >> ofi, r"DefaultGroupName=%s" % self.name
        print >> ofi, r"OutputBaseFilename=%s-%s-setup" % (self.name, self.version)
        print >> ofi
        print >> ofi, r"[Files]"
        for path in self.windows_exe_files + self.lib_files:
            print >> ofi, r'Source: "%s"; DestDir: "{app}\%s"; Flags: ignoreversion' % (path, os.path.dirname(path))
        print >> ofi
        print >> ofi, r"[Icons]"
        for path in self.windows_exe_files:
            print >> ofi, r'Name: "{group}\%s"; Filename: "{app}\%s"' % \
                  (self.name, path)
        print >> ofi, 'Name: "{group}\Uninstall %s"; Filename: "{uninstallexe}"' % self.name

    def compile(self, compilercmd="compile"):
        """compiles the script using InnoSetup"""
        try:
            import ctypes
        except ImportError:
            try:
                import win32api
            except ImportError:
                os.startfile(self.pathname)
            else:
                print "Ok, using win32api."
                win32api.ShellExecute(0, compilercmd, self.pathname, None, None, 0)
        else:
            print "Cool, you have ctypes installed."
            res = ctypes.windll.shell32.ShellExecuteA(0, compilercmd, self.pathname, None, None, 0)
            if res < 32:
                raise RuntimeError, "ShellExecute failed, error %d" % res

###############################################################

class build_installer(build_exe):
    """distutils class that first builds the exe file(s), then creates a Windows installer using InnoSetup"""
    def run(self):
        # First, let py2exe do it's work.
        build_exe.run(self)
        lib_dir = self.lib_dir
        dist_dir = self.dist_dir
        # create the Installer, using the files py2exe has created.
        script = InnoScript(self.distribution.metadata.name, lib_dir, dist_dir, self.windows_exe_files, self.lib_files, version=wxPRE.getversion())
        print "*** creating the inno setup script***"
        script.create()
        print "*** compiling the inno setup script***"
        script.compile("compil32")
        # Note: By default the final setup.exe will be in an Output subdirectory.

class wxPREDistribution(Distribution):
    """A distribution class for wxPRE"""
    def __init__(self, attrs):
      baseattrs = {}
      baseattrs['script_name'] = os.path.basename(sys.argv[0])
      baseattrs['script_args'] = sys.argv[1:]
      # package information
      baseattrs['name'] = "wxPRE"
      baseattrs['version'] = wxPRE.__version__
      baseattrs['url'] = "http://wiki.wxpython.org/index.cgi/wxPRE"
      baseattrs['author'] = "David Fraser and Stephen Emslie"
      baseattrs['author_email'] = 'info@sjsoft.com'
      baseattrs['description'] = 'wxPython Runtime Environment'
      baseattrs['long_description'] = wxPRE.__doc__
      baseattrs['license'] = "wxWidgets license"
      py2exeoptions = {}
      py2exeoptions["packages"] = ["wx"]
      py2exeoptions["compressed"] = True
      py2exeoptions["dist_dir"] = "wxPRE-%s" % wxPRE.getversion()
      baseattrs['options'] = {"py2exe": py2exeoptions}
      if py2exe:
          baseattrs['console'] = ["wxPRE.py"]
          baseattrs['zipfile'] = "wxPRElibs.zip"
      baseattrs['cmdclass'] = {"py2exe": build_installer}
      baseattrs.update(attrs)
      Distribution.__init__(self, baseattrs)


if __name__ == "__main__":
    dist = wxPREDistribution({})
    try:
        ok = dist.parse_command_line()
    except DistutilsArgError, msg:
        raise SystemExit, gen_usage(dist.script_name) + "\nerror: %s" % msg
    if ok:
        try:
            dist.run_commands()
        except KeyboardInterrupt:
            raise SystemExit, "interrupted"
        except (IOError, os.error), exc:
            error = grok_environment_error(exc)
            raise SystemExit, error
        except (DistutilsError, CCompilerError), msg:
            raise SystemExit, "error: " + str(msg)